import 'package:flutter/material.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

class SeniorCheckbox extends StatelessWidget {
  final String? title;
  final bool value;
  final bool disabled;
  final double extraTapMargin;
  final bool actionOnTitle;
  final Function(bool?)? onChanged;

  const SeniorCheckbox({
    Key? key,
    this.title,
    required this.value,
    required this.onChanged,
    this.actionOnTitle = false,
    this.extraTapMargin = 0,
    this.disabled = false,
  }) : super(key: key);

  static const double size = Checkbox.width;

  void _handleValueChange() {
    if (!disabled && onChanged != null) {
      onChanged!(!value);
    }
  }

  Widget _buildCheckboxControl() {
    return Container(
      width: SeniorCheckbox.size,
      height: SeniorCheckbox.size,
      decoration: BoxDecoration(
        color: SeniorColors.pureWhite,
        border: Border.all(
          color: disabled
              ? SeniorColors.secondaryColor300
              : value
                  ? SeniorColors.primaryColor
                  : SeniorColors.secondaryColor600,
          width: 1,
        ),
        borderRadius: BorderRadius.circular(3),
      ),
      child: Theme(
        data: ThemeData(
          unselectedWidgetColor: Colors.transparent,
          disabledColor: Colors.transparent,
        ),
        child: Checkbox(
          value: value,
          onChanged: null,
          activeColor: Colors.transparent,
          checkColor: disabled
              ? SeniorColors.secondaryColor300
              : SeniorColors.primaryColor,
          materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return ListTileTheme.merge(
      child: Stack(
        alignment: Alignment.centerLeft,
        children: [
          GestureDetector(
            onTap: actionOnTitle ? _handleValueChange : null,
            child: Row(
              children: [
                SizedBox(
                  width: extraTapMargin,
                ),
                Padding(
                  padding: const EdgeInsets.only(
                    right: SeniorSpacing.xsmall,
                  ),
                  child: _buildCheckboxControl(),
                ),
                title != null
                    ? Text(
                        title!,
                        style: SeniorTypography.text(
                          color: disabled
                              ? SeniorColors.secondaryColor300
                              : SeniorColors.neutralColor900,
                        ),
                      )
                    : SizedBox.shrink(),
              ],
            ),
          ),
          GestureDetector(
            onTap: _handleValueChange,
            child: Container(
              color: Colors.transparent,
              width: extraTapMargin + size + extraTapMargin,
              height: extraTapMargin + size + extraTapMargin,
            ),
          ),
        ],
      ),
    );
  }
}
