import re

class ChangelogReader:
    def __init__(self):
        self.issue_pattern = re.compile('\*?\s*?\[#?(?P<key>\w+-\d+)\](?P<link>\(http[s]?:\/\/[^)]*\))?\s*-?\s*(?P<summary>.*)', re.MULTILINE)

    def read_issues(self, changelog_path, tag):
        issues = []
        tag = tag.replace('v','').replace('-','.')
        tag_section_pattern = re.compile('^# [0-9\.]+')
        with open(changelog_path,'r') as changelog:
            line = changelog.readline()
            is_in_tag_section = False
            while line:
                if tag in line:
                    is_in_tag_section = True
                    line = changelog.readline()
                    continue
                
                started_other_section = is_in_tag_section and tag_section_pattern.match(line)
                if started_other_section:
                    break

                if is_in_tag_section and self.issue_pattern.match(line):
                    issues.append(self.issue_from(line))

                line = changelog.readline()
                
            changelog.close()

        return issues    

    def issue_from(self, line):
        matches = self.issue_pattern.search(line)
        return Issue(key=matches.group('key'), 
                    summary=matches.group('summary'), 
                    link=matches.group('link')
               )

class Issue:
    def __init__(self, key, summary, link):
        self.key = key
        self.summary = summary
        self.link = link

    def __str__(self):
        return 'Issue {self.key} {self.summary} {self.link}'.format(self=self)