import sys
import getopt

sys.path.append("./senior-ci")

from common import (print_message, exit_message, get_env_variable)
from colorama import Fore

from jira import Jira, Issue, IssueLink
from changelog_reader import ChangelogReader

class CreateSreIssue:
    def __init__(self, argv):
        print_message(argv)
        self.changelog_path = './CHANGELOG.md'
        self.jira           = Jira('https://jira.senior.com.br/rest/api/2', get_env_variable('JIRA_USER'), get_env_variable('JIRA_PASSWORD'))
        self.tag            = get_env_variable('CI_COMMIT_REF_SLUG')
        self.repository     = get_env_variable('CI_PROJECT_NAME')
        self.module         = ''
        self.sre_team       = ''
        
        try:
            opts, _ = getopt.getopt(argv, ['t:r:m:c:s:'], ['tag=', 'repository=', 'module=', 'changelog=','sre_team='])
            for opt, arg in opts:
                if opt in ('-t', '--tag'):
                    self.tag = arg
                elif opt in ('-r', '--repository'):
                    self.repository = arg
                elif opt in ('-m', '--module'):
                    self.module = arg
                elif opt in ('-c', '--changelog'):
                    self.changelog_path = arg
                elif opt in ('-s', '--sre_team'):
                    self.sre_team = arg

            if not self.sre_team:
                raise Exception('O parâmetro --sre_team é obrigatório')

        except Exception as err:
            exit_message(err)

    def exec(self):
        """Reads the issues from a given version in changelog and creates an 'deploy issue'"""
        solved_issues = self.__read_changelog()
        if not solved_issues:
            print_message("Nenhuma task encontrada no CHANGELOG para a tag "+ self.tag, Fore.LIGHTYELLOW_EX)
            return
            
        sre_issue_type = 'Bug' if any(issue.type == 'Bug' for issue in solved_issues) else 'Story'
        img_docker_custom_field = 'customfield_12002' if 'frontend' in self.repository else 'customfield_12001'
        sre_issue = self.__create_sre_issue(sre_issue_type, img_docker_custom_field)
        print_message('Task criada para o SRE: {key} - {summary} ({tag})'.format(key = sre_issue.key, 
                                                                                 summary = sre_issue.summary, 
                                                                                 tag = self.tag), Fore.GREEN)
        for issue in solved_issues:
            if self.jira.create_link(IssueLink(issue, sre_issue, 'Dependência entre atividades')):
                print_message("Link criado: " + issue.key +" - " + sre_issue.key, Fore.GREEN)

    def __read_changelog(self):
        changelog_issues = ChangelogReader().read_issues(self.changelog_path, self.tag)
        jira_issues = []
        for changelog_issue in changelog_issues:
            jira_issues.append(self.jira.get_issue(changelog_issue.key))
        return jira_issues

    def __create_sre_issue(self, sre_issue_type, img_docker_custom_field):
        return self.jira.create_issue(Issue(
            key = '',
            customfields = {
                img_docker_custom_field: [ self.repository + ':' + self.tag ],
                'customfield_11100': [{ 'value': 'Produção - platform.senior.com.br'}]
            },
            type = 'Deploy', 
            summary = 'Deploy ' + self.repository, 
            project_key = self.sre_team,
            labels = [sre_issue_type, self.module]
        ))