import requests
from requests.auth import HTTPBasicAuth
from os import path

class Jira:
    def __init__(self, url, user, password):
        self.url = url
        self.auth = HTTPBasicAuth(user, password)
        requests.packages.urllib3.disable_warnings() 

    def create_issue(self, issue):
        fields = {
            'project': {
                'key': issue.project_key
            },  
            'summary': issue.summary,
            'issuetype': {
                'name': issue.type
            },
            'labels' : issue.labels
        }

        data = {
            'fields': {**fields, **issue.customfields}
        }
        
        resp = requests.post(
            url=path.join(self.url,'issue'), 
            auth=self.auth, 
            headers={'Content-Type':'application/json'}, 
            verify=False, 
            json=data
        )
        return self.get_issue(resp.json()['key'])
        
    def create_link(self, link):
        resp = requests.post(
            url=path.join(self.url,'issueLink'), 
            auth=self.auth, 
            headers={'Content-Type':'application/json'}, 
            verify=False, 
            json={
                    'outwardIssue': {
                        'key': link.outward_issue.key
                    },
                    'inwardIssue': {
                        'key': link.inward_issue.key
                    },
                    'type': {
                        'name': link.type
                    }
                }
            )
        return resp.ok

    def get_issue(self, key):
        resp = requests.get(url=path.join(self.url, 'issue', key), auth=self.auth)
        issue_json = resp.json()
        return Issue(key = issue_json['key'], 
                    type = issue_json['fields']['issuetype']['name'], 
                    summary = issue_json['fields']['summary'], 
                    project_key = issue_json['fields']['project']['key'],
                    labels = issue_json['fields']['labels']
        )

    def __str__(self):
        return self.key    

class Issue:
    def __init__(self, key, type, summary, project_key, labels, customfields = []):
        self.key = key
        self.type = type
        self.summary = summary
        self.labels = labels
        self.project_key = project_key
        self.customfields = customfields

class IssueLink:
    """A link between two issues"""
    def __init__(self, inward_issue, outward_issue, type):
        self.inward_issue = inward_issue
        self.outward_issue = outward_issue
        self.type = type